/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Timo Bingmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Timo Bingmann <timo.bingmann@student.kit.edu>
 */
#include "ns2ext-error-rate-model.h"
#include "wifi-phy.h"
#include "ns3/log.h"

NS_LOG_COMPONENT_DEFINE ("Ns2ExtErrorRateModel");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (Ns2ExtErrorRateModel);

TypeId 
Ns2ExtErrorRateModel::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::Ns2ExtErrorRateModel")
    .SetParent<ErrorRateModel> ()
    .AddConstructor<Ns2ExtErrorRateModel> ()
    ;
  return tid;
}

Ns2ExtErrorRateModel::Ns2ExtErrorRateModel ()
{}

double 
Ns2ExtErrorRateModel::GetSuccessRate (WifiMode mode, double snr) const
{
  /* Values from ns-2.33 with patch
| mode  | SINR_dB | SINR_ratio |
|-------+---------+------------|
| BPSK  |     5.0 |     3.1623 |
| QPSK  |     8.0 |     6.3096 |
| QAM16 |    15.0 |    31.6228 |
| QAM64 |    25.0 |   316.2278 |
  */
    
  if (mode == WifiPhy::Get6mba ())
    {
      return (snr >= 3.1623) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get9mba ())
    {
      return (snr >= 3.1623) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get12mba ())
    {
      return (snr >= 6.3096) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get18mba ())
    {
      return (snr >= 6.3096) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get24mba ())
    {
      return (snr >= 31.6228) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get36mba ())
    {
      return (snr >= 31.6228) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get48mba ())
    {
      return (snr >= 316.2278) ? 1.0 : 0.0;
    }
  else if (mode == WifiPhy::Get54mba ())
    {
      return (snr >= 316.2278) ? 1.0 : 0.0;
    }

  NS_ASSERT_MSG(false, "Unconfigured WifiMode or incompatible with this ErrorRateModel.");
  return 0;
}

double 
Ns2ExtErrorRateModel::GetChunkSuccessRate (WifiMode mode, double snr, uint32_t nbits) const
{
  NS_ASSERT_MSG(false, "Not implemented");
  return 0.0;
}

} // namespace ns3
